/*
 * Copyright(C) OASIS(R) 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 */
package client_bjm;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.Properties;

import javax.jms.BytesMessage;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import client.BaseJAXWSTestRunner;

/**
 * A generic test client based on JMS APIs.  
 * Extends the BaseJAXWSTestRunner for common setup/teardown and configuration.
 * Overrides the invokeTest method to use JMS instead.
 */
public class BaseJMSTestRunner extends BaseJAXWSTestRunner {

    private static String JNDI_PROPERTIES_FILE_NAME = "/jndi.properties";
    private static String JNDI_NAMING_PROVIDER_URL_PROPNAME = "java.naming.provider.url";
    private static String JNDI_NAMING_FACTORY_INITIAL_PROPNAME = "java.naming.factory.initial";
    protected InitialContext ctx;
    private String serviceResponse = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><return>service1 operation1 invoked with <TESTNAME> request message</return>";

    /**
     * Invoke the test via a JMS message exchange.
     * @param input - input string
     * @return the output string from the JMS message exchange.
     * @throws Exception - if the JMS message exchange fails (e.g. the target service does not exist)
     */
    public String invokeTest( String input ) throws Exception {

        String output = "No response";

        // Get JNDI properties and ensure they are set as required.
        Properties properties = new Properties();
        try {
            InputStream propertiesStream = this.getClass().getResourceAsStream(JNDI_PROPERTIES_FILE_NAME);
            properties.load(propertiesStream);
        } catch (IOException e) {	
            System.out.println("Unable to read JNDI properties file");
        } // end try

        // Correct test name in serviceResponse
        serviceResponse = serviceResponse.replace("<TESTNAME>", testConfiguration.testName);

        // Get the name of the JMS Destination for this test case.
        String destinationName = ((JMSTestConfiguration)testConfiguration).getDestinationName();
        String connectionFactoryName = ((JMSTestConfiguration)testConfiguration).getConnectionFactoryName();

        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, properties.getProperty(JNDI_NAMING_FACTORY_INITIAL_PROPNAME));
        env.put(Context.PROVIDER_URL, properties.getProperty(JNDI_NAMING_PROVIDER_URL_PROPNAME));
 
        ctx = new InitialContext(env);

        ConnectionFactory connFactory = (javax.jms.ConnectionFactory) ctx.lookup(connectionFactoryName);
        Destination dest = (javax.jms.Destination)ctx.lookup(destinationName);
        
        output = invokeTest(input,connFactory,dest);
     
        return output;
    }

    /**
     * Invoke the test via a JMS message exchange.
     * @param input - input string
     * @param 
     * @return the output string from the JMS message exchange.
     * @throws Exception - if the JMS message exchange fails (e.g. the target service does not exist)
     */
    public String invokeTest( String input, ConnectionFactory connFactory, Destination dest ) throws Exception {
        String output = "No response";

        // Send a text message.
        Connection conn = connFactory.createConnection();
        Session sess = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);

        Message request = createRequestMessage(input,sess);

        sendRequestMessage(sess,dest,request);

        if (((JMSTestConfiguration)testConfiguration).mockServiceDestinationName != null) {
            handleMockService(connFactory);
        }

        // Receive the reply if one is expected.
        if( request.getJMSReplyTo() != null ) {
            MessageConsumer msgConsumer = sess.createConsumer(request.getJMSReplyTo());
            conn.start();
            Message response = msgConsumer.receive();

            output = checkReplyMessage(response);
        }

        sess.close();
        conn.close();

   	return output;
    } // end method invokeTest    

    /**
     * Create the request message.
     * @param input - input string
     * @param sess - the JMS session
     * @return the request message.
     * @throws Exception - if the JMS message exchange fails (e.g. the target service does not exist)
     */
    public Message createRequestMessage( String input, Session sess ) throws Exception {
        Destination replyQueue;
    	if (((JMSTestConfiguration)testConfiguration).getReplyDestinationName() != null) {
    		replyQueue = (javax.jms.Destination)ctx.lookup(((JMSTestConfiguration)testConfiguration).getReplyDestinationName());
    	} else {
            replyQueue = sess.createTemporaryQueue();
    	}
        TextMessage textMsg = sess.createTextMessage();
        textMsg.setJMSReplyTo(replyQueue);
        textMsg.setText("<?xml version=\"1.0\" encoding=\"UTF-8\"?><operation1>"+input+"</operation1>");

        return textMsg;
    }

    /**
     * Send the request message.
     * @param sess - the JMS session
     * @param dest - the JMS destination
     * @param message - the JMS message
     *
     * @throws Exception - if the JMS message exchange fails (e.g. the target service does not exist)
     */
    public void sendRequestMessage( Session sess, Destination dest, Message request ) throws Exception {
        MessageProducer msgProducer = sess.createProducer(dest);
        msgProducer.send(request);
    }

    protected void handleMockService(ConnectionFactory connFactory) throws NamingException, JMSException {
        Destination dest = (javax.jms.Destination)ctx.lookup(((JMSTestConfiguration)testConfiguration).mockServiceDestinationName);
        Connection conn = connFactory.createConnection();
        conn.start();
        Session sess = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
        MessageConsumer consumer = sess.createConsumer(dest);
        Message m = consumer.receive();
        checkServiceRequest(m);
        sendServiceReply(m, sess);
        sess.close();
        conn.stop();
        conn.close();
    }

    protected void checkServiceRequest(Message m) throws JMSException {
    }

    protected void sendServiceReply(Message request, Session sess) throws JMSException {
        MessageProducer msgProducer = sess.createProducer(request.getJMSReplyTo());
        TextMessage response = sess.createTextMessage();
        response.setText(serviceResponse);

        String correlationScheme = ((JMSTestConfiguration)testConfiguration).mockServiceCorrelationScheme;
        if ("correlationID".equals(correlationScheme)) {
            response.setJMSCorrelationID(request.getJMSCorrelationID());
        } else if ("messageID".equals(correlationScheme)) {
            response.setJMSCorrelationID(request.getJMSMessageID());
        }
        msgProducer.send(response);
        msgProducer.close();
    }

    /**
     * Check the reply message.
     *
     * @param reply - the reply message
     * @return the output.
     * @throws Exception - if the JMS message exchange fails (e.g. the target service does not exist)
     */
    public String checkReplyMessage( Message reply ) throws Exception {
        String output = "No reply";

        if( reply instanceof TextMessage) {
            output = ((TextMessage)reply).getText();
        } else if( reply instanceof BytesMessage) {
            byte[] bs = new byte[(int)((BytesMessage)reply).getBodyLength()];
            ((BytesMessage)reply).readBytes(bs);
            output = new String(bs);
        } else if (reply instanceof ObjectMessage) {
            Object o = ((ObjectMessage)reply).getObject();
            if (o instanceof Exception) {
            	throw (Exception)o;
            }
            output = o.toString();
        } else {
            System.out.println("Reply is "+reply);
        }

        return output;
    }


    
} // end class BaseJMSTestRunner
